@@ -9,6 +9,7 @@ from textwrap import dedent
 from typing import Optional
 
 import click
+import git
 from kapitan.resources import inventory_reclass
 
 from commodore.config import Config
@@ -21,6 +22,7 @@ from commodore.dependency_mgmt.jsonnet_bundler import fetch_jsonnet_libraries
 from commodore.helpers import kapitan_compile, relsymlink, yaml_dump
 from commodore.inventory import Inventory
 from commodore.inventory.lint import check_removed_reclass_variables
+from commodore.multi_dependency import MultiDependency
 from commodore.postprocess import postprocess_components
 
 
@@ -32,6 +34,8 @@ def compile_component(
     value_files_: Iterable[str],
     search_paths_: Iterable[str],
     output_path_: str,
+    repo_directory_: str,
+    component_name: str,
 ):
     # Resolve all input to absolute paths to fix symlinks
     component_path = P(component_path_).resolve()
@@ -39,9 +43,18 @@ def compile_component(
     search_paths = [P(d).resolve() for d in search_paths_]
     search_paths.append(component_path / "vendor")
     output_path = P(output_path_).resolve()
+    if repo_directory_:
+        repo_directory = P(repo_directory_).resolve()
+        sub_path = str(component_path.relative_to(repo_directory))
+    else:
+        # If no repo directory given, assume component path is repo root
+        repo_directory = component_path
+        sub_path = ""
 
-    # Ignore 'component-' prefix in dir name
-    component_name = component_path.stem.replace("component-", "")
+    if not component_name:
+        # Ignore 'component-' prefix in repo name, this assumes that the repo name
+        # indicates the component name for components in subpaths.
+        component_name = repo_directory.stem.replace("component-", "")
 
     # Fall back to `component as component` when instance_name is empty
     if instance_name_ is None or instance_name_ == "":
@@ -62,7 +75,11 @@ def compile_component(
         inv.ensure_dirs()
         search_paths.append(inv.dependencies_dir)
         component = _setup_component(
-            config, component_name, instance_name, component_path
+            config,
+            component_name,
+            instance_name,
+            repo_directory,
+            sub_path,
         )
         _prepare_kapitan_inventory(inv, component, value_files, instance_name)
 
@@ -115,9 +132,21 @@ def compile_component(
 
 
 def _setup_component(
-    config: Config, component_name: str, instance_name: str, component_path: P
+    config: Config,
+    component_name: str,
+    instance_name: str,
+    repo_directory: P,
+    sub_path: str,
 ) -> Component:
-    component = Component(component_name, directory=component_path)
+    if not repo_directory.is_dir():
+        raise click.ClickException(
+            f"Can't compile component, repository {repo_directory} doesn't exist"
+        )
+    cr = git.Repo(repo_directory)
+    cdep = MultiDependency(cr.remote().url, config.inventory.dependencies_dir)
+    component = Component(
+        component_name, dependency=cdep, directory=repo_directory, sub_path=sub_path
+    )
     config.register_component(component)
     config.register_component_aliases({instance_name: component_name})
 
